@@ -1,6 +1,6 @@
 # vim:cindent:ts=2:sw=2:et:fdm=marker:cms=\ #\ %s
 #
-# $Id: Scrollkeeper.pm 157 2008-11-11 13:21:09Z robert $
+# $Id: Scrollkeeper.pm 223 2011-02-28 23:34:22Z robert $
 #
 
 package Debian::DocBase::Programs::Scrollkeeper;
@@ -11,20 +11,22 @@ use warnings;
 
 use vars qw(@ISA @EXPORT);
 @ISA = qw(Exporter);
-@EXPORT = qw(RegisterScrollkeeper);
+@EXPORT = qw(RegisterScrollkeeper ScrollkeeperStatusChanged 
+             $SC_NOTCHANGED $SC_INSTALLED $SC_REMOVED);
 
-use Carp;
 use Debian::DocBase::Common;
 use Debian::DocBase::Utils;
 use Debian::DocBase::Gettext;
-use File::Basename qw(dirname);
+use Debian::DocBase::DB;
 use UUID;
 
 
 
+our $SC_NOTCHANGED             = 0;
+our $SC_INSTALLED              = 1;
+our $SC_REMOVED                = 2;
 
 our $scrollkeeper_update       = "/usr/bin/scrollkeeper-update";
-#our $scrollkeeper_gen_seriesid = "/usr/bin/scrollkeeper-gen-seriesid";
 our $scrollkeeper_map_file     = "/usr/share/doc-base/data/scrollkeeper.map";
 
 
@@ -59,12 +61,37 @@ sub _GetUUID() { # {{{
 sub RegisterScrollkeeper($@) { # {{{
   my $showinfo  = shift;
   my @documents = @_;
+
+  if (! $opt_update_menus)
+  {
+    Debug(_g("Skipping registration of %s because of --no-update-menus."), $opt_update_menus);
+    return;
+  }
+
+  my $statusChanged=ScrollkeeperStatusChanged();
+  if (-x $scrollkeeper_update)
+  {
+    Inform(_g("Registering documents with %s..."), "scrollkeeper") if $showinfo;
+    _RegisterScrollkeeperFiles(@documents);
+  }
+  elsif ($statusChanged == $SC_REMOVED)
+  {
+    Inform(_g("Unregistering documents from %s..."), "scrollkeeper") if $showinfo;
+    _UnregisterScrollkeeperFiles(@documents);
+  }
+  else
+  {
+    Debug(_g("Skipping execution of %s - %s package doesn't seem to be installed."), $scrollkeeper_update, "rarian-compat");
+  }
+  _SaveScrollkeeperStatus() if $statusChanged != $SC_NOTCHANGED;
+} # }}}
+
+
+sub _RegisterScrollkeeperFiles($@) { # {{{
+  my @documents = @_;
   my $do_update = 0;
 
-  Inform(_g("Registering documents with %s..."), "scrollkeeper")
-    if $showinfo and $opt_update_menus and -x $scrollkeeper_update;
-
-  Debug(_g("%s started"), "RegisterScrollkeeper");
+  Debug(_g("%s started."), "_RegisterScrollkeeperFiles");
 
   # read in doc-base -> scrollkeeper mappings unless already read
   ReadMap($scrollkeeper_map_file, \%mapping);
@@ -83,6 +110,9 @@ sub RegisterScrollkeeper($@) { # {{{
         next unless defined $format_data;
 
         my $file = defined $$format_data{'index'} ? $$format_data{'index'} : $$format_data{'files'};
+        next if $file =~ /\n/; # avoid `Unsuccessful stat on filename containing newline'
+                               # given by perl in the next line when the files field is a
+                               # multiline one, see Bug#607498
         next unless -f $file;
 
         $omf_serial_id = $doc->GetStatus('Scrollkeeper-sid');
@@ -108,12 +138,25 @@ sub RegisterScrollkeeper($@) { # {{{
 
   Execute($scrollkeeper_update, '-q') if ($do_update and $opt_update_menus);
 
-
-  Debug(_g("%s finished"), "RegisterScrollkeeper");
+  Debug(_g("%s finished."), "_RegisterScrollkeeperFiles");
 } # }}}
 
 
+sub _UnregisterScrollkeeperFiles($@) { # {{{
+  my @documents = @_;
 
+  Debug(_g("%s started."), "_UnRegisterScrollkeeperFiles");
+  foreach my $doc (@documents) {
+    my $old_omf_file = $doc->GetStatus('Scrollkeeper-omf-file');
+    next unless defined $old_omf_file;
+    my $omf_serial_id = $doc->GetStatus('Scrollkeeper-sid');
+    _RemoveOmfFile($old_omf_file);
+    $doc->SetStatus( 'Scrollkeeper-omf-file' => undef,
+                     'Scrollkeeper-sid'      =>  $omf_serial_id);
+  }
+  Debug(_g("%s finished."), "_UnRegisterScrollkeeperFiles");
+
+} # }}}
 
 # arguments: filename
 # reads a file that looks like:
@@ -126,16 +169,23 @@ sub _MapDocbaseToScrollkeeper($) { # {{{
   return $mapping{lc($_[0])};
 } # }}}
 
+sub _DirName { # {{{
+    my @p = split '/', $_[0];
+    return (join '/', @p[0..($#p-1)]) if $#p > 1;
+    return '/' if substr ($_[0], 0, 1) eq '/';
+    return '.';
+} # }}}
+
 sub _RemoveOmfFile($) { # {{{
   my $omf_file = shift;
-  my $omf_dir = dirname($omf_file);
-  Debug( _g("Removing scrollkeeper OMF file `%s'"), $omf_file);
-  unlink($omf_file) or return Error( _g("%s: could not delete file: %s"), $omf_file, $!);
+  my $omf_dir = _DirName($omf_file);
+  Debug( _g("Removing scrollkeeper OMF file `%s'."), $omf_file);
+  unlink($omf_file) or return Error( _g("Cannot remove file `%s': %s."), $omf_file, $!);
 
   #check to see if the directory is now empty. if so, kill it.
   if (opendir(DIR, $omf_dir)) {
     if (defined grep { $_ !~ /^\.\.?$/ } readdir DIR) {
-      rmdir($omf_dir) or Error( _g("Could not delete directory `%s': %s"), $omf_dir, $!);
+      rmdir($omf_dir) or Error( _g("Cannot delete directory `%s': %s."), $omf_dir, $!);
     }
     closedir DIR;
   }
@@ -161,7 +211,7 @@ sub _WriteOmfFile($$$$) { # {{{
 
 
   if (! -d "$OMF_DIR/$docid") {
-    mkdir("$OMF_DIR/$docid") or croak "Cannot create dir `$OMF_DIR/$docid': $!";
+    mkdir("$OMF_DIR/$docid") or Fatal($ERR_FSACCESS, _g("Cannot create directory `%s': %s."), $OMF_DIR/$docid, $!);
   }
 
   &Debug("Writing scrollkeeper OMF file `$omf_file'");
@@ -186,9 +236,32 @@ sub _WriteOmfFile($$$$) { # {{{
 
   #finish the boiler plate
   print OMF "\t</resource>\n</omf>\n";
-  close(OMF) or croak "Cannot close OMF file `$omf_file': $!";
+  close(OMF) or Fatal($ERR_FSACCESS, _g("Cannot close file `%s': %s."), $omf_file, $!);
 
   return $omf_file;
 } # }}}
 
+sub ScrollkeeperStatusChanged() # {{{
+{
+  my $scStatus = Debian::DocBase::DB::GetStatusDB()->GetData("/internal/sc-status");
+  unless (defined $scStatus)
+  {
+    _SaveScrollkeeperStatus();
+    return 0;
+  }
+  my $statusChanged = (($scStatus ? 1 : 0) xor (-x $scrollkeeper_update ? 1 : 0));
+  Debug(_g("Scrollkeeper status changed: %d."), $statusChanged);
+  return $SC_NOTCHANGED unless $statusChanged;
+  return $SC_INSTALLED  if -x $scrollkeeper_update;
+  return $SC_REMOVED;
+
+  
+} # }}}
+
+sub _SaveScrollkeeperStatus() # {{{
+{
+  my $status = (-x $scrollkeeper_update ? 1 : 0);
+  Debian::DocBase::DB::GetStatusDB()->PutData("/internal/sc-status", $status);
+  Debug(_g("Scrollkeeper status set to %d."), $status);
+} # }}}
 1;
